/*
     PS2 KEYBOARD COMMAND SET
*/

#ifndef PS2_KbdProtocol_H
#define	PS2_KbdProtocol_H

#ifdef	__cplusplus
extern "C" {
#endif

                            /* *() Originally available in PS/2 keyboards only. */
    
    
#define KBD_RESET_CMD 0xFF  /**< (Reset) - Keyboard responds with "ack" (0xFA), then enters "Reset" mode. (See "Reset" section.) */

#define KBD_RESEND_CMD 0xFE /**< (Resend) - Keyboard responds by resending the last-sent byte. The exception to this is if the last-sent byte
                                            was "resend" (0xFE). If this is the case, the keyboard resends the last non-0xFE byte.
                                            This command is used by the host to indicate an error in reception.
                                            The next six commands can be issued when the keyboard is in any mode, but it only effects the behavior of
											the keyboard when in "mode 3" (ie, set to scan code set 3.)
                                            The PS/2 Keyboard Interface http://www.computer-engineering.org/ps2keyboard */

#define KBD_SET_KEY_TYPE_MAKE_CMD 0xFD       /**< *(Set Key Type Make) - Disable break codes and typematic repeat for specified keys. Keyboard responds
  								                                         with "ack" (0xFA), then disables scanning (if enabled) and reads a list of keys from the host. These keys are
 								                                         specified by their set 3 make codes. Keyboard responds to each make code with "ack". Host terminates this list by
								                                         sending an invalid set 3 make code (eg, a valid command.) The keyboard then re-enables scanning (if previously	disabled). */

#define KBD_MAKE_BREAK_CMD 0xFC                 /**< *(Set Key Type Make/Break) - Similar to previous command, except this one only disables typematic repeat. */

#define KBD_SET_KEY_TYPE_CMD 0xFB               /**< *(Set Key Type Typematic) - Similar to previous two, except this one only disables break codes. */

#define KBD_TELEMATIC_SET_MAKE_BREAK_CMD 0xFA   /**< *(Set All Keys Typematic/Make/Break) - Keyboard responds with "ack" (0xFA). Sets all keys to their
                                                                                         normal setting (generate scan codes on make, break, and typematic repeat) */

#define KBD_SET_ALL_KEYS_MAKE_CMD 0xF9          /**< *(Set All Keys Make) - Keyboard responds with "ack" (0xFA). Similar to 0xFD, except applies to all keys. */
#define KBD_SET_ALL_KEYS_MAKE_BREAK_CMD 0xF8    /**< *(Set All Keys Make/Break) - Keyboard responds with "ack" (0xFA). Similar to 0xFC, except applies to all keys. */

#define KBD_ALL_KEYS_TELEMATIC_CMD 0xF7         /**< *(Set All Keys Typematic) - Keyboard responds with "ack" (0xFA). Similar to 0xFB, except applies to all keys. */

#define KBD_SET_DEFAULT_CMD 0xF6                /**< (Set Default) - Load default typematic rate/delay (10.9cps / 500ms), key types (all keys
                                                                     typematic/make/break), and scan code set (2). */

#define KBD_DISABLE_CMD 0xF5                    /**< (Disable) - Keyboard stops scanning, loads default values (see "Set Default" command), and waits further instructions. */

#define KBD_ENABLE_CMD 0xF4                     /**< (Enable) - Re-enables keyboard after disabled using previous command. */

#define KBD_TELEMATIC_SET_RATE_CMD 0xF3         /**< (Set Typematic Rate/Delay) - Host follows this command with one argument byte that defines the typematic
                                                                                  rate and delay as follows: */
#define KBD_READ_ID_CMD   0xF2                  /**< *(Read ID) - The keyboard responds by sending a two-byte device ID of 0xAB, 0x83. (0xAB is sent first, followed by 0x83.) */
        
#define KBD_SET_SCAN_CODE_SET_CMD 0xF0          /**< *(Set Scan Code Set) - Keyboard responds with "ack", then reads argument byte from the host. This
                                                                            argument byte may be 0x01, 0x02, or 0x03 to select scan code set 1, 2, or 3, respectively. The keyboard responds
                                                                            to this argument byte with "ack". If the argument byte is 0x00, the keyboard responds with "ack" followed by the
                                                                            current scan code set. */
        
#define KBD_ECHO_CMD 0xEE                       /**< (Echo) - The keyboard responds with "Echo" (0xEE).*/
        
#define KBD_SET_RESET_LEDS_CMD    0xED          /**< (Set/Reset LEDs) - The host follows this command with one argument byte, that specifies the state of the
                                                                        keyboard's Num Lock, Caps Lock, and Scroll Lock LEDs. This argument byte is defined as follows: */
    
/******************************************************************************************/    
/**< BITS of KBD_SET_RESET_LEDS_CMD */
#define PS2_KBD_CAPS_LOCK_BIT    0x04     /**<  "Caps Lock" - Caps Lock LED off(0)/on(1) */
#define PS2_KBD_NUM_LOCK_BIT     0x02     /**< "Num Lock" - Num Lock LED off(0)/on(1) */
#define PS2_KBD_SCROLL_LOCK_BIT  0x01     /**< "Scroll Lock" - Scroll Lock LED off(0)/on(1) */
/******************************************************************************************/    
    
/*    
Initialization:
  
The following is the communication between my computer and keyboard when it boots-up. I beleive the first three
commands were initiated by the keyboad controller, the next command (which enables Num lock LED) was sent by the
BIOS, then the rest of the commands were sent my the OS (Win98SE). Remember, these results are specific to my
computer, but it should give you a general idea as to what happens at startup.
Keyboard: AA Self-test passed ;Keyboard controller init
  
        Host:      ED Set/Reset Status Indicators
        Keyboard:  FA Acknowledge
        Host:      00 Turn off all LEDs
        Keyboard:  FA Acknowledge
        Host:      F2 Read ID
        Keyboard:  FA Acknowledge
        Keyboard:  AB First byte of ID
        Host:      ED Set/Reset Status Indicators ;BIOS init
        Keyboard:  FA Acknowledge
        Host:      02 Turn on Num Lock LED
        Keyboard:  FA Acknowledge
        Host:      F3 Set Typematic Rate/Delay ;Windows init
        Keyboard:  FA Acknowledge
        
        
        Host:      20 500 ms / 30.0 reports/sec
        Keyboard:  FA Acknowledge
        Host:      F4 Enable
        Keyboard:  FA Acknowledge
        Host:      F3 Set Typematic Rate/delay
        Keyboard:  FA Acknowledge
        Host:      00 250 ms / 30.0 reports/sec
        Keyboard:  FA Acknowledge    
*/
    
#ifdef	__cplusplus
}
#endif

#endif
// EOF