/**
  Generated main.c file from MPLAB Code Configurator

  @Company
    Microchip Technology Inc.

  @File Name
    main.c

  @Summary
   
    This is the generated main.c 
  
  @Description
   
   	Implementation of PS2 / USB keyboard converter (c) 2020 JW Technical Consultancy Ltd

    Author                        : JW Technical Consultancy Ltd
  
    TARGET MCU                    : PIC24FJ128GB202
    MPLAB 	                      : MPLAB X v5.40
    MPLAB Code Configurator (MCC) : Version 4.0
    Compiler                      : XC16 v1.5
    Heap allocated                : 4096 bytes
  
    Initally attempts to locate a PS2 keyboard and if not found after a couple of seconds
    tries to find a USB keyboard.   
    I was experimenting with making these "Plug and Play", which is a work in progress.
  
*/

/**
  Section: Included Files
*/
#include "mcc_generated_files/system.h"
#include "mcc_generated_files/pin_manager.h"
#include "mcc_generated_files/interrupt_manager.h"
#include <stdbool.h>
#include <stdio.h>
#include <stdlib.h>
#include "global.h"
#include "mcc_generated_files/usb/usb.h"
#include "Timer.h"
#include "PS2_keyboard.h"
#include "USB_keyboard.h"
#include "kbd_parallel_out.h"


const char SIGN_ON_MSG[] = "\r\nUSB-PS2 keyboard converter V2.9\r\n";    

#define PS2_ALLOWED_DETECT_TIME_S                     3   /**< Time allowed to detect PS2 keyboard before trying USB keyboard */
#define RETRY_ALL_TIME_S                              10  /**< Total time to search for eithier type of keyboard befor repeating */
#define SEND_SIGN_ON_MESSAGE_TO_PARALLEL_PORT      false  /**< Enable/disable sign on message */ 
#define SEND_KEYBOARD_DETECTION_MESSAGE_TO_SERIAL   true  /**< Enable/disable keyboard reporting to serial port */
#define INITIALISE_DELAY_uS                        15000  /**< Initialisation delay in uS. PS2 Keyboard needed 1 second to initialise */
#define ONE_SECOND_TIME_uS                 ONE_SECOND_uS  /**< 1 second in uS's */ 

extern uint8_t  ascii_code;
extern uint8_t  lastKey; 
extern uint8_t  keyReleased;
extern uint32_t keyTimer;
extern uint32_t keyRepeatTime;

uint32_t elapsedTime_mS(uint32_t since);

/*
                         Main application
 */
int main(void)
{
static bool        Initialised = false;
static uint32_t    StartUpTimer;
static uint32_t    SecondsCounter;
static uint32_t    OnSecondTimer = 0;
static bool        PS2_keyboardDetected = false;
static bool        startUSB_Poll = false;
static bool        usbKeyboardWasAttached = false;

    // initialize the device
    SYSTEM_Initialize();
    
     // Initialise 100uS timer
    Timer_uS_Init();
    
    // Enable the Global Interrupts
    INTERRUPT_GlobalEnable(); 

    // TODO Handle watchdog for this MCU 
    
    //if(!STATUSbits.nTO)  /* check for WDT reset occurrence */
    //{
    //    printf("\r\nWDT Reset!\r\n");
    //}

    // Enable PS2 keyboard interface
    EnablePS2Keyboard();
    
    // Sign on message to serial port
    printf(SIGN_ON_MSG);
    
    APP_HostHIDKeyboardInitialize();
            
    // Initialise timers
    StartUpTimer = OnSecondTimer = GetTimer_uS();
    
    SecondsCounter = 0;
    
    while (1)
    {
        
        // If UART overrun, reset receiver
        if (U1STAbits.OERR)
        {
            U1STAbits.URXEN = 0;
            U1STAbits.URXEN = 1;
        }
        
        // Sign on message
        if ((Initialised == false) && (elapsedTime_mS(StartUpTimer) >= INITIALISE_DELAY_uS))
        {
            StartUpTimer = GetTimer_uS();
            #if SEND_SIGN_ON_MESSAGE_TO_PARALLEL_PORT == true
            SendErrorMsgToHost(SIGN_ON_MSG, ONLY_PARALLEL_PORT);
            #endif
            Initialised = true;
        }

        // ********************************************************************
        // One second timer
        // ********************************************************************
        if ( elapsedTime_mS(OnSecondTimer) >= ONE_SECOND_TIME_uS )
        {
            OnSecondTimer = GetTimer_uS();
            SecondsCounter++; 
            
            if (PS2KeyboardConnected() && (!PS2_keyboardDetected))
            {
                PS2_keyboardDetected = true;
                #if SEND_KEYBOARD_DETECTION_MESSAGE_TO_SERIAL == true
                printf("PS2 keyboard initialised in %u seconds\r\n", SecondsCounter);
                #endif
            }
            
            // If not detected PS2 keyboard, try to find USB keyboard
            
            if ((PS2_keyboardDetected == false) && (!startUSB_Poll) && (SecondsCounter == PS2_ALLOWED_DETECT_TIME_S))
            {
                startUSB_Poll = true;    // Enable USB keyboard poll 
                //#if SEND_KEYBOARD_DETECTION_MESSAGE_TO_SERIAL == true
                //puts("USB polling\r\n");
                //#endif
                DisablePS2Keyboard();    // Disable PS2 interface
            }
                    
            // If no keyboards detected after a few seconds, retry 
            if ((PS2_keyboardDetected == false) && (APP_HostHIDKeyboardConnected() == false) && (SecondsCounter == RETRY_ALL_TIME_S))
            {
                // puts("Retry PS2 and USB\r\n");
                // Re-initialise PS2 keyboard interface
                EnablePS2Keyboard();
                PS2_keyboardDetected = false;
                // Re-initialise USB keyboard detection
                USBInitialize();
                APP_HostHIDKeyboardInitialize();
                usbKeyboardWasAttached = false;  
                startUSB_Poll = false;
                // Reset timer
                SecondsCounter = 0;
            }
        }

        // ********************************************************************
        // Poll and support USB keyboard
        // ********************************************************************
        if ((PS2KeyboardConnected() == false) && (startUSB_Poll == true))
        {
            USBHostTasks();
            USBHostHIDTasks();
            APP_HostHIDKeyboardTasks();
            CodeOutputTask();
            
            if (APP_HostHIDKeyboardConnected() && (usbKeyboardWasAttached == false))
            {
                usbKeyboardWasAttached = true;  
            }
            
            if ((APP_HostHIDKeyboardConnected() == false) && (usbKeyboardWasAttached == true))
            {
                #if SEND_KEYBOARD_DETECTION_MESSAGE_TO_SERIAL == true
                puts("USB keyboard detached\r\n");   
                #endif
                // Re-initialise PS2 keyboard interface
                EnablePS2Keyboard();
                PS2_keyboardDetected = false;
                // Re-initialise USB keyboard detection
                USBInitialize();
                APP_HostHIDKeyboardInitialize();
                usbKeyboardWasAttached = false;  
                startUSB_Poll = false;
                // Reset timer
                SecondsCounter = 0;
            }
        }
        // ********************************************************************
        // Poll and support PS2 keyboard
        // ********************************************************************
        if ((Initialised) && (startUSB_Poll == false))
        {
            PS2KeyboardTask(); // Key board decode
            
            // If PS2 keyboard disconnects
            if ((PS2KeyboardConnected() == false) && (PS2_keyboardDetected))
            {
                #if SEND_KEYBOARD_DETECTION_MESSAGE_TO_SERIAL == true
                puts("USB keyboard detached\r\n");   
                #endif      
                // Re-initialise PS2 keyboard interface
                EnablePS2Keyboard();
                PS2_keyboardDetected = false;
                // Re-initialise USB keyboard detection
                USBInitialize();
                APP_HostHIDKeyboardInitialize();
                usbKeyboardWasAttached = false;  
                startUSB_Poll = false;
                // Reset timer
                SecondsCounter = 0;
            }
        }

        		/* Auto repeat key strokes from USB keyboard */
		if ( (elapsedTime_mS(keyTimer) >= keyRepeatTime) && (keyReleased == false))
		{
			keyTimer = GetTimer_uS();
			keyRepeatTime = KEY_REPEAT_TIME_uS;
            
            if ( ascii_code != 0 )
            {
                SendCharToHost(lastKey);
            }
		}
        
        CodeOutputTask();

        /* Clear Watchdog Timer */
    }
    
    return 1; 
}


/*
 * @brief Return difference in timer count
 * @detail
 * */
uint32_t elapsedTime_mS(uint32_t since)
{
	uint32_t difference = 0;
	uint32_t IC_Val1 = since;
	uint32_t IC_Val2 = GetTimer_uS();

	if (IC_Val2 > IC_Val1)
	{
		difference = IC_Val2-IC_Val1;
	}
	else if (IC_Val1 > IC_Val2)
	{
		difference = (0xffffffff - IC_Val1) + IC_Val2;
	}
	else
	{
		difference = 0;
	}

	return difference;
}

/**
 End of File
*/

